#! /usr/bin/perl -wT
################################################################################
#
# Transfer Form Frame for Gold Web GUI
#
# File   :  pretransfer.cgi
# History:  27 JUL 2005 [Scott Jackson] first implementation
#
################################################################################
#                                                                              #
#                              Copyright (c) 2005                              #
#                  Pacific Northwest National Laboratory,                      #
#                         Battelle Memorial Institute.                         #
#                             All rights reserved.                             #
#                                                                              #
################################################################################
#                                                                              #
#     Redistribution and use in source and binary forms, with or without       #
#     modification, are permitted provided that the following conditions       #
#     are met:                                                                 #
#                                                                              #
#      Redistributions of source code must retain the above copyright         #
#     notice, this list of conditions and the following disclaimer.            #
#                                                                              #
#      Redistributions in binary form must reproduce the above copyright      #
#     notice, this list of conditions and the following disclaimer in the      #
#     documentation and/or other materials provided with the distribution.     #
#                                                                              #
#      Neither the name of Battelle nor the names of its contributors         #
#     may be used to endorse or promote products derived from this software    #
#     without specific prior written permission.                               #
#                                                                              #
#     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS      #
#     "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        #
#     LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS        #
#     FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE           #
#     COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,      #
#     INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,     #
#     BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;         #
#     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER         #
#     CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT       #
#     LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN        #
#     ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE          #
#     POSSIBILITY OF SUCH DAMAGE.                                              #
#                                                                              #
################################################################################

use strict;
use vars qw();
use lib qw(/usr/lib /usr/lib/perl5);
use CGI qw(:standard);
use CGI::Session;
use XML::LibXML;
use Gold::CGI;
use Gold::Global;

my $cgi = new CGI;
my $session = new CGI::Session(undef, $cgi, { Directory => "/tmp" });
my $username = $session->param("username");
my $password = $session->param("password");
unless ($username && $password)
{
  print header;
  print start_html(-title => "Session Expired",
                   -onLoad => "top.location.replace(\"login.cgi\");"
  );
  print end_html;
  exit(0);
}

my $object = $cgi->param("myobject");
my $action = $cgi->param("myaction");

my $style_sheet = <<END_STYLE_SHEET;
<!--
        \@import "/usr/share/gold-2.1.12.2/styles/gold.css";
                                                                                
        body {
            background-color: white;
        }
                                                                                
        h1 {
            font-size: 110%;
            text-align: center;
        }

        .include {
            width: 100%;
        }

        .include {
            float: left;
        }


        .include select{
            width: 100%;
        }

        #includeHead, #excludeHead {
            border-bottom: 1px solid black;
            margin-top: 10px;
            margin-bottom: 2px;
            font-weight: bold;
            font-size: 110%;
        }

        html&gt;body #includeHead, html&gt;body #excludeHead {
            margin-top: 0px;
        }

-->
END_STYLE_SHEET

# Javascript to changeActivationTable and getValue
my $java_script = <<END_JAVA_SCRIPT;
function submitForm()
{
  if(document.getElementById('FromId').value == "")
    alert("A Source Account Id must be specified.  Please specify a From Account Id and press the Make Transfer button again.");
  else if(document.getElementById('ToId').value == "")
    alert("A Destination Account Id must be specified.  Please specify a To Account Id and press the Make Transfer button again.");
  else if(document.getElementById('Amount').value == "")
    alert("An Amount must be specified.  Please specify an amount and press the Make Transfer button again.");
  else
  {
    document.inputForm.submit();
  }
}

function newWindow(windowName, url, winWidth, maxHeight)
{
  var winHeight = ((screen.availHeight-100) > maxHeight) ? maxHeight : screen.availHeight-100;
                                                                                
  var nw=window.open(url, windowName, "scrollbars,status,resizable,width=" + winWidth + ",height=" + winHeight);
  if(!nw) return null;
                                                                                
  if(parseInt(navigator.appVersion) >= 4)
  {
    var nwLeft = parseInt((screen.availWidth - winWidth) / 2);
    var nwTop = parseInt((screen.availHeight - winHeight) / 2);                                                                                 
    if(parseInt(navigator.appVersion) >= 4)
    {
      nw.moveTo(nwLeft, nwTop-50);
    }
  }
  nw.focus();
  return nw;
}

  var whichId;
  function setId(id){
    document.getElementById(whichId).value = id;
  }
        
  function selectAccount(toOrFrom){
    whichId = toOrFrom;
    newWindow('Select Account', 'selectAccount.cgi', 750, 500);
  }
END_JAVA_SCRIPT

# Print the create form
print header;
print start_html(-title => "Make Transfer",
                 -script => { -code => $java_script },
                 -style => { -code => $style_sheet },
);
print_body();
print end_html;

sub print_body
{
  # Print the header
  print div( { -class => "header" },
    div( { -class => "leftFlare" },
      img( { -alt => "",
             -height => "36",
             -width => "16",
             -src => "/usr/share/gold-2.1.12.2/images/header_flare_left.gif" } ) ),
    div( { -class => "rightFlare" },
      img( { -alt => "",
             -height => "36",
             -width => "16",
             -src => "/usr/share/gold-2.1.12.2/images/header_flare_right.gif" } ) ),
    h1("Make Transfer") );

  # Print text boxes
  print <<END_OF_SCREEN_TOP;
<p xmlns="" id="requiredDesc">Fields marked with a red asterisk (<span class="required">*</span>) are required.</p>
<form xmlns="" name="inputForm" method="post" action="transfer.cgi">
  <input type="hidden" name="myobject" value="$object">
  <input type="hidden" name="myaction" value="$action">
  <div class="row">
    <label for="FromId" class="rowLabel">
    <span class="required">*</span>From Account Id:</label>
    <input type="text" name="FromId" id="FromId" value="" size="30" maxlength=""/>
    <input type="button" name="selectFromAccountButton" id="selectFromAccountButton" value="Select..." onClick="selectAccount('FromId')"/>
  </div>
  <div class="row">
    <label for="ToId" class="rowLabel">
    <span class="required">*</span>To Account Id:</label>
    <input type="text" name="ToId" id="ToId" value="" size="30" maxlength=""/>
    <input type="button" name="selectToAccountButton" id="selectToAccountButton" value="Select..." onClick="selectAccount('ToId')"/>
  </div>
  <div class="row">
    <label for="Amount" class="rowLabel">
    <span class="required">*</span>Account Amount:</label>
    <input type="text" name="Amount" id="Amount" value="" size="30" maxlength=""/>
  </div>
  <div class="row">
    <label for="Allocation" class="rowLabel">Allocation Id:</label>
    <input type="text" name="Allocation" id="Allocation" value="" size="30" maxlength=""/>
  </div>
  <div class="row">
    <label for="Description" class="rowLabel">Description:</label>
    <input type="text" name="Description" id="Description" value="" size="30" maxlength=""/>
  </div>
  <hr/>
  <div id="submitRow">
    <input onClick="submitForm()" value="Make Transfer" id="formSubmit" type="button" name="transfer"/>
  </div>
</form>
END_OF_SCREEN_TOP
}


